#! /usr/bin/python3

class Messages:
    fileNotFound = "{0}: Ce fichier ou ce dossier n'existe pas."
    fileAlreadyExists = "{0}: {1}: Le fichier existe déjà."
    blogName = "Nom du blog"
    yourName = "Votre nom"
    blogDescription = "Courte description du blog"
    blogKeywords = "Quelques mots clefs qui définissent votre blog"
    aboutYou = "À propos de vous"
    license = "La licence appliquée à votre contenu"
    blogUrl = "L'url du blog"
    blogLanguage = "La langue du blog"
    yourEmail = "Votre adresse email"
    missingParams = "{0}: Paramètres manquants."
    cannotReadIn = "{0}: Impossible de lire dans {1}."
    nothingToDo = "Rien à faire."
    unknownCommand = "{0}: Commande inconnue."
    noBlogConfiguration = "Le fichier de configuration du blog n'existe pas ou vous n'avez pas les bonnes permissions."
    missingMandatoryFieldInBlogConf = "Attention, le champ \"{0}\" est manquant dans le fichier de configuration principal du blog."
    doNotForgetToIncludeCSSFileInHeader= "N'oubliez pas d'inclure le fichier CSS {0} dans header.html."
    username="Nom d'utilisateur: "
    userPasswd="Mot de passe utilisateur: "
    ftpHost="Nom d'hôte FTP"
    ftpPath="Chemin absolu de votre blog sur l'hôte FTP."
    cleanFtpDirectory="Nettoyage du repertoire FTP de destination..."
    copyToFtpDirectory="Copie du blog vers le repertoire FTP de destination..."
    possibleMalformedEntry="La publication {0} est probablement mal formée... Abandon."
    blogRecompilation="Recompilation du blog locale..."
    exportArchives="Exportation locale du fil de publications de '{0}'."
    exportMainThread="Exportation locale du fil de publications principal."
    exportMainThreadRss="Exportation locale du flux RSS du fil de publications principal."
    exportCategories="Exportation locale du fil de publications de la categorie '{0}'."
    exportCategoriesRss="Exportation locale du flux RSS du fil de publications de la categorie '{0}'."
    missingMandatoryFieldInEntry="Le champ '{0}' dans l'entrée numéro {1} est manquant."
    getUnknownValue="Get: La valeur {0} n'existe pas."
    forUnknownValue="For: La valeur {0} n'existe pas."
    recursiveForUnknownValue="RecursiveFor: La valeur {0} n'existe pas."
    notEnoughArgs="Paramétres manquants."
    unknownPattern="Le motif '{0}' n'existe pas."
    unknownContextual="La variable contextuel {0} n'existe pas."
    inRessource="Dans la ressource '{0}'."
    somethingGoesWrongReturnEmptyString="Une erreur s'est produite. Remplacement pas une chaine de caractère vide."
    argBlogName="Nom du blog {0}"
    argEntryName="Nom de la publication"
    argTemplateName="Nom du template"
    argInputFilename="Nom de fichier"
    blogFolderDoesntExists="Le dossier de destination n'existe pas... VenC s'en fiche et va en faire un nouveau."



#! /usr/bin/python3

def PrintCategoriesTree(tree,indentation=""):
    for node in tree:
        print(indentation+node.value, "->", node.relatedTo)
        PrintCategoriesTree(node.childs,indentation+"\t")
        



#! /usr/bin/python3

import os
import yaml
import codecs
import datetime
import subprocess

import VenC.core
import VenC.pattern

def entry(argv):
    if len(argv) < 1:
        print("VenC: "+VenC.core.Messages.missingParams.format("--new-entry"))
        return
    
    if VenC.core.blogConfiguration == None:
        print("VenC: "+VenC.core.Messages.noBlogConfiguration)
        return
    
    content =   {"authors":	"",
		"tags":		"",
		"categories":	"",
                "CSS":     ""}
    try:
        wd = os.listdir(os.getcwd())
    except OSError:
        print(VenC.core.Messages.cannotReadIn.format(os.getcwd()))
        return

    date = datetime.datetime.now()

    entry = VenC.core.SetNewEntryMetadata(date, argv[0])

    content["entry_name"] = argv[0]
    entryDate = str(date.month)+'-'+str(date.day)+'-'+str(date.year)+'-'+str(date.hour)+'-'+str(date.minute)
    outputFilename = os.getcwd()+'/entries/'+str(entry["EntryID"])+"__"+entryDate+"__"+content["entry_name"].replace(' ','_')
    if len(argv) == 1:
        stream = codecs.open(outputFilename,'w',encoding="utf-8")
        output = yaml.dump(content, default_flow_style=False, allow_unicode=True) + "---\n"
        stream.write(output)
    else:
        try:
            output = open(os.getcwd()+'/templates/'+argv[1], 'r').read()
            patternProcessor = VenC.pattern.processor(".:",":.","::")
            currentData = VenC.core.GetPublicDataFromBlogConf()
            currentData["EntryName"] = argv[0]
            currentData["EntryDate"] = VenC.core.GetFormattedDate(entryDate)
            patternProcessor.SetWholeDictionnary(currentData)
            patternProcessor.ressource = '/templates/'+argv[1]
            output = patternProcessor.parse(output)
            stream = codecs.open(outputFilename,'w',encoding="utf-8")
            stream.write(output)

        except FileNotFoundError as e:
            print("VenC: "+VenC.core.Messages.fileNotFound.format(os.getcwd()+"/templates/"+argv[1]))
            return
    stream.close()
    try:
        pass
        subprocess.call([VenC.core.blogConfiguration["textEditor"], outputFilename]) 
    except:
        pass

def blog(argv):
    default_configuration =	{"blog_name":			VenC.core.Messages.blogName,
                                "textEditor":                   "nano",
                                "date_format":                  "%A %d. %B %Y",
				"author_name":			VenC.core.Messages.yourName,
				"blog_description":		VenC.core.Messages.blogDescription,
				"blog_keywords":		VenC.core.Messages.blogKeywords,
				"author_description":		VenC.core.Messages.aboutYou,
				"license":			VenC.core.Messages.license,
				"url":				VenC.core.Messages.blogUrl,
                                "ftp_host":                     VenC.core.Messages.ftpHost,
				"blog_language":		VenC.core.Messages.blogLanguage,
				"email":			VenC.core.Messages.yourEmail,
				"path":				{"ftp":                         VenC.core.Messages.ftpPath,
                                                                "index_file_name":		"index{page_number}.html",
								"category_directory_name":	"{category}",
								"dates_directory_name":		"%Y-%m",
								"entry_file_name":		"entry{entry_id}.html",
								"rss_file_name":		"feed.xml"},
				"entries_per_pages":		10,
				"columns":			1,
				"rss_thread_lenght":		5,
				"thread_order":			"latest first"}
    for folder_name in argv:
        try:
            os.mkdir(folder_name)
        except OSError:
            print("VenC: "+VenC.core.Messages.fileAlreadyExists.format("--new-blog",os.getcwd()+'/'+folder_name))
            return
        os.mkdir(folder_name+'/'+"blog")
        os.mkdir(folder_name+'/'+"entries")
        os.mkdir(folder_name+'/'+"theme")
        os.mkdir(folder_name+'/'+"extra")
        os.mkdir(folder_name+'/'+"templates")
        stream = codecs.open(folder_name+'/'+'blog_configuration.yaml', 'w',encoding="utf-8")
        yaml.dump(default_configuration, stream, default_flow_style=False, allow_unicode=True)



#! /usr/bin/python3

class Messages:
    fileNotFound = "{0}: This file or directory does not exists."
    fileAlreadyExists = "{0}: {1}: File or directory already exists."
    blogName = "Blog name"
    yourName = "Your name"
    blogDescription = "Short description of your blog"
    blogKeywords = "Some keywords about your blog"
    aboutYou = "About you"
    license = "The license applied to your blog"
    blogUrl = "The URL of your blog"
    blogLanguage = "The language of your blog"
    yourEmail = "Your e-mail"
    missingParams = "{0}: Missing arguments."
    cannotReadIn = "{0}: Cannot read into {1}."
    nothingToDo = "Nothing to do."
    unknownCommand = "{0}: Unknown command."
    noBlogConfiguration = "Blog's configuration file doesn't exists or you do not have right permissions."
    missingMandatoryFieldInBlogConf = "Warning, the field \"{0}\" is missing the blog's configuration file."
    doNotForgetToIncludeCSSFileInHeader= "Do not forget to include CSS file {0} in header.html."
    username="Username: "
    userPasswd="User password: "
    ftpHost="FTP hostname"
    ftpPath="Absolute path of your blog on FTP server."
    cleanFtpDirectory="Cleaning target FTP directory..."
    copyToFtpDirectory="Copying your blog to target FTP directory..."
    possibleMalformedEntry="Possible malformed entry {0}. Abort."
    blogRecompilation="Recompilation of your blog locally..."
    exportArchives="Exporting local entries thread of '{0}'."
    exportMainThread="Exporting local main entries thread."
    exportMainThreadRss="Exporting local main RSS feed entries."
    exportCategories="Exporting local categories entries thread named '{0}'."
    exportCategoriesRss="Exporting local entries RSS feed thread from category '{0}'."
    missingMandatoryFieldInEntry="Field '{0}' in entry number {1} is missing."
    getUnknownValue="Get: Value {0} isn't defined."
    forUnknownValue="For: Value {0} isn't defined."
    recursiveForUnknownValue="RecursiveFor: Value {0} isn't defined."
    notEnoughArgs="Missing parameters."
    unknownPattern="The pattern '{0}'doesn't exists."
    unknownContextual="Contextual variable {0} doesn't exists."
    inRessource="In ressource '{0}'."
    somethingGoesWrongReturnEmptyString="An error occurs. Replacement with an empty string."
    argBlogName="Blog name {0}"
    argEntryName="Entry name"
    argTemplateName="Template name"
    argInputFilename="Filename"
    blogFolderDoesntExists="Destination folder doesn't exist. VenC doesn't care and will built a new one."



#! /usr/bin/python3

import os
import time
import math
import yaml
import base64
import datetime
import pygments
import VenC.pattern
import pygments.lexers
import pygments.formatters

errors=list()

class OutputColors:
    FAIL = '\033[91m'
    END     = '\033[0m'

def GetMessages():
    import locale
    currentLocale = locale.getlocale()[0].split('_')[0]
    if currentLocale == 'fr':
        from VenC.languages import fr as language
    else:
        from VenC.languages import en as language
    return language.Messages()

Messages = GetMessages()

def CodeHighlight(argv):
    try:
        lexer = pygments.lexers.get_lexer_by_name(argv[0], stripall=True)

        formatter = pygments.formatters.HtmlFormatter(linenos=("inline" if argv[1]=="True" else False),cssclass="venc_source_"+argv[0])
        code = base64.b64decode(bytes(argv[2],encoding='utf-8'))
        result = pygments.highlight(code, lexer, formatter)
        css  = formatter.get_style_defs('.venc_source_'+argv[0])
    
        if not os.path.exists(os.getcwd()+"/extra/venc_source_"+argv[0]+".css"):
            print(Messages.doNotForgetToIncludeCSSFileInHeader.format("venc_source_"+argv[0]+".css"))
            stream = open(os.getcwd()+"/extra/venc_source_"+argv[0]+".css",'w')
            stream.write(css)

        return result

    except Exception as e:
        raise
        print("VenC:", e)
        return str()

class Key:
    def __init__(self, value, entry):
        self.count = 1
        self.weight = 1
        self.path = str()
        self.value = value
        self.relativeOrigin = str()
        self.relatedTo = [entry]
        self.childs = list()


class Theme:
    def __init__(self):
        self.header = str()
        self.footer = str()
        self.entry = str()
        self.rssHeader = str()
        self.rssFooter = str()
        self.rssEntry = str()

        try:
            self.header = open(os.getcwd()+"/theme/chunks/header.html",'r').read()
            self.footer = open(os.getcwd()+"/theme/chunks/footer.html",'r').read()
            self.entry = open(os.getcwd()+"/theme/chunks/entry.html",'r').read()
            self.rssHeader = open(os.getcwd()+"/theme/chunks/rssHeader.html",'r').read()
            self.rssFooter = open(os.getcwd()+"/theme/chunks/rssFooter.html",'r').read()
            self.rssEntry = open(os.getcwd()+"/theme/chunks/rssEntry.html",'r').read()

        except FileNotFoundError as e:
            print("VenC: "+Messages.fileNotFound.format(str(e.filename)))
            exit()

def GetConfigurationFile():
    try:
        blogConfiguration = yaml.load(open(os.getcwd()+"/blog_configuration.yaml",'r').read())
        
        mandatoryFields = [
            "blog_name",
            "textEditor",
            "date_format",
	    "author_name",
	    "blog_description",
	    "blog_keywords",
	    "author_description",
	    "license",
	    "url",
            "ftp_host",
	    "blog_language",
	    "email",
	    "entries_per_pages",
            "columns",
	    "rss_thread_lenght",
            "thread_order"
        ]

        everythingIsOkay = True
        for field in mandatoryFields:
            if not field in blogConfiguration.keys():
                everythingIsOkay = False
                print("VenC: "+Messages.missingMandatoryFieldInBlogConf.format(field))
        
        mandatoryFields = [
            "index_file_name",
	    "category_directory_name",
	    "dates_directory_name",
	    "entry_file_name",
	    "rss_file_name",
            "ftp"
        ]

        for field in mandatoryFields:
            if not field in blogConfiguration["path"].keys():
                everythingIsOkay = False
                print("VenC: "+Messages.missingMandatoryFieldInBlogConf.format(field))

        if not everythingIsOkay:
            return -1

        return blogConfiguration

    except:
        return None

blogConfiguration = GetConfigurationFile()
if blogConfiguration == -1:
    exit()

def orderableStrToInt(string):
    try:
        return int(string)

    except:
        return -1

def GetLatestEntryID():
    entriesList = sorted(GetEntriesList(), key = lambda entry : int(entry.split("__")[0]))
    
    if len(entriesList) != 0:
        return int(entriesList[-1].split("__")[0])
    else:
        return 0

def GetEntriesList():
    try:
        files = os.listdir(os.getcwd()+"/entries")

    except FileNotFoundError as e:
        print("VenC: "+Messages.fileNotFound.format(os.getcwd()+"/entries"))
        exit()
    
    entries = dict()
    for filename in files:
        explodedFilename = filename.split("__")
        try:
            date = explodedFilename[1].split('-')
            entryID = int(explodedFilename[0])
            datetime.datetime(year=int(date[2]),month=int(date[0]),day=int(date[1]),hour=int(date[3]),minute=int(date[4])) 
            if entryID > 0:
                entries[filename] = open(os.getcwd()+"/entries/"+filename,'r').read()
        except ValueError:
            pass

        except IndexError:
            pass

    return entries

def GetSortedEntriesList(inputEntries):
    return sorted(inputEntries, key = lambda e : int(e.split("__")[0]), reverse=(blogConfiguration["thread_order"].strip() == "latest first"))

def MergeDictionnary(current,public):
    d = current.copy()
    d.update(public)
    return d

def GetPublicDataFromBlogConf():
    data = dict()
    data["AuthorName"] = blogConfiguration["author_name"]
    data["BlogName"] = blogConfiguration["blog_name"]
    data["BlogDescription"] = blogConfiguration["blog_description"]
    data["BlogKeywords"] = blogConfiguration["blog_keywords"]
    data["AuthorDescription"] = blogConfiguration["author_description"]
    data["License"] = blogConfiguration["license"]
    data["BlogUrl"] =blogConfiguration["url"]
    data["BlogLanguage"] = blogConfiguration["blog_language"]
    data["AuthorEmail"] = blogConfiguration["email"]
    return data

def SetNewEntryMetadata(entryDate, entryName):
    entry = dict()
    entry["EntryID"] = GetLatestEntryID()+1
    entry["EntryName"] = entryName
    entry["EntryMonth"] = entryDate.month
    entry["EntryYear"] = entryDate.year
    entry["EntryDay"] = entryDate.day
    entry["EntryHour"] = entryDate.hour
    entry["EntryMinute"] = entryDate.minute

    publicDataFromBlogConf = GetPublicDataFromBlogConf()
    for key in publicDataFromBlogConf:
        entry[key] = publicDataFromBlogConf[key]  

    return entry
    
def PrintVersion(argv):
    print("VenC 1.2.0")

def GetKeyByName(keys, name):
    for key in keys:
        if key.value == name:
            return key
    return None

def GetEntriesPerDates(entries):
    entriesPerDates = list()
    for entry in entries.keys():
        date = time.strftime(blogConfiguration["path"]["dates_directory_name"], time.strptime(entry.split("__")[1],"%m-%d-%Y-%M-%S"))
        try:
            selectedKey = GetKeyByName(entriesPerDates, date)
            selectedKey.relatedTo.append(entry)
            selectedKey.count+=1
        except:
            entriesPerDates.append(Key(date,entry))

    return entriesPerDates

def GetDatesList(keys, relativeOrigin):
    output = list()
    maxWeight = 0
    for key in keys:
        if maxWeight < key.count:
            maxWeight = key.count
        output.append({"date": key.value, "count":key.count,"dateUrl":relativeOrigin+key.value})

    for key in output:
        key["weight"] = str(int((key["count"]/maxWeight)*10))

    return sorted(output, key = lambda date: datetime.datetime.strptime(date["date"], blogConfiguration["path"]["dates_directory_name"])) 

def GetCategoriesList(entries):
    output = list()
    for entry in entries.keys():
        stream = entries[entry].split("---\n")[0]
        data = yaml.load(stream)
        if data != None:
            for category in data["categories"].split(","):
                if (not category in output) and category != '':
                    output.append(category)

    return output

def GetEntriesPerCategories(entries):
    entriesPerCategories = list()
    for entry in entries.keys():
        stream = entries[entry].split("---\n")[0]
        try:
            data = yaml.load(stream)
        except yaml.scanner.ScannerError:
            print("VenC:", VenC.core.Messages.possibleMalformedEntry.format(entry))
            exit()
        except yaml.parser.ParserError:
            print("VenC:", VenC.core.Messages.possibleMalformedEntry.format(entry))
            exit()

        if data != None:
            try:
                for category in data["categories"].split(","):
                    if category != '':
                        nodes = entriesPerCategories
                        path = str()
                        for subCategory in category.split(" > "):
                            path += subCategory+'/'
                            try:
                                selectedKey = GetKeyByName(nodes, subCategory.strip())
                                selectedKey.relatedTo.append(entry)
                                selectedKey.count += 1
                                selectedKey.relatedTo = list(set(selectedKey.relatedTo))
                            except Exception as e:
                                selectedKey = Key(subCategory.strip(), entry)
                                selectedKey.path = path
                                nodes.append(selectedKey)
                            
                            nodes = selectedKey.childs
                
            except TypeError:
                print("VenC:", VenC.core.Messages.possibleMalformedEntry.format(entry))
                exit()

    return entriesPerCategories 

def GetCategoriesTreeMaxWeight(categories, maxWeight=0):
    currentMaxWeight = maxWeight
    for category in categories:
        if category.count > currentMaxWeight:
            currentMaxWeight = category.count
        m = GetCategoriesTreeMaxWeight(category.childs, maxWeight=currentMaxWeight)
        if m > currentMaxWeight:
            currentMaxWeight = m

    return currentMaxWeight

def GetCategoriesTree(categories, relativeOrigin, root, maxWeight=None):
    node = root

    if len(categories) != 0:
        node["_nodes"] = dict()

    for category in categories:
        node[category.value] = dict()
        node[category.value]["__categoryPath"] = category.path
        if maxWeight != None:
            node[category.value]["__count"] = category.count
            node[category.value]["__weight"] = int((category.count/maxWeight) * 10)
        node[category.value]["__relativeOrigin"] = relativeOrigin
        if len(category.childs) != 0:
            node[category.value]["_nodes"] = dict()
            GetCategoriesTree(category.childs, relativeOrigin, node[category.value]["_nodes"], maxWeight)
    return node  

def ToBase64_(argv):
    return "~§CodeHighlight§§"+argv[0]+"§§"+argv[1]+"§§"+base64.b64encode(bytes(argv[2],encoding='utf-8')).decode("utf-8", "strict")+"§~"

def GetFormattedDate(unformattedDate):
    data = unformattedDate.split('-')
    return datetime.datetime(
        year=int(data[2]),
        month=int(data[0]),
        day=int(data[1]),
        hour=int(data[3]),
        minute=int(data[4])
    ).strftime(blogConfiguration["date_format"])

def GetListOfPages(entriesPerPage,entriesCount):
    listOfPages = list()
    pagesCount = math.ceil(entriesCount/entriesPerPage)
    for pageNumber in range(0,pagesCount):
        listOfPages.append(
            {
                "pageNumber": pageNumber,
                "pageUrl": "index"+str(pageNumber)+".html" if pageNumber != 0 else "index.html" 
            }
        )
    return listOfPages



#! /usr/bin/python3

import cgi
import VenC.core

class processor():
    def __init__(self, openSymbol, closeSymbol, separator):
        self.closeSymbol	= closeSymbol
        self.openSymbol		= openSymbol
        self.separator		= separator
        self.dictionnary        = dict()
        self.functions		= dict()
        self.functions["Get"] = self.Get
        self.functions["For"] = self.For
        self.functions["RecursiveFor"] = self.RecursiveFor
        self.strict             = True
        self.currentStrings     = dict()
        self.currentString      = str()
        self.ressource          = str()
        self.preProcessedStrings = dict()
        self.patternsIndex      = dict()
        self.errors             = list()

    def handleError(self,error, default,enable=False):
        if self.strict or enable:
            err = VenC.core.OutputColors.FAIL+"VenC: "+error+"\n"
            
            if self.ressource != str():
            	err +="VenC: "+VenC.core.Messages.inRessource.format(self.ressource)+"\n"
                
            if not err in VenC.core.errors:
                VenC.core.errors.append(err)
                err += self.currentString+VenC.core.OutputColors.END+"\n\n"
                print(err)
        
        return default

    def SetFunction(self, key, function):
        self.functions[key] = function

    def DelFunction(self, key):
        try:
            del self.functions[key]
        except:
            pass

    def DelValue(self, key):
        try:
            del self.dictionnary[key]
        except:
            pass

    def SetWholeDictionnary(self, dictionnary):
        for key in dictionnary:
            self.dictionnary[key] = dictionnary[key]

    def Set(self, symbol, value):
       self.dictionnary[symbol] = value

    def Get(self, symbol):
        try:
            return self.dictionnary[symbol[0]]

        except KeyError as e:
            return self.handleError(VenC.core.Messages.getUnknownValue.format(e), "~§"+"Get§§"+"$$".join(symbol)+"§~")

        except IndexError as e:
            return self.handleError("Get: "+VenC.core.Messages.notEnoughArgs.format(e), "~§"+"Get§§"+"$$".join(symbol)+"§~",True)

    def For(self, argv):
        outputString = str()
        try:
            for Item in self.dictionnary[argv[0]]:
                outputString += argv[1].format(Item) + argv[2]

            return outputString[:-len(argv[2])]
        except KeyError as e:
            return self.handleError(VenC.core.Messages.forUnknownValue.format(e),"~§For§§"+"§§".join(argv)+"§~",True)
        
        except IndexError as e:
            return self.handleError("For: "+VenC.core.Messages.notEnoughArgs,"~§For§§"+"§§".join(argv)+"§~",True)

    def _RecursiveFor(self, openString, content, separator,closeString, nodes):
        outputString = openString
        try:
            for Key in sorted(nodes.keys()):
                variables = dict()
                for key in nodes[Key]:
                    if key[:2] == '__':
                        variables[key[2:]] = nodes[Key][key]
                variables["item"] = Key
                if Key != "_nodes":
                    if not "_nodes" in nodes[Key].keys():
                        outputString += content.format(variables) + separator
                    else:
                        outputString += content.format(variables) + self._RecursiveFor(openString, content, separator, closeString, nodes[Key]["_nodes"])
        except KeyError as e:
            return self.handleError(VenC.core.Messages.forUnknownValue.format(e),"<!-- Recursive For KeyError exception, shouldn't happen -->",True)

        return outputString + closeString

    def RecursiveFor(self, argv):
        outputString = str()
        try:
            outputString += self._RecursiveFor(
                argv[1],
                argv[2],
                argv[3],
                argv[4],
                self.dictionnary[argv[0]]
            )
            return outputString
        except IndexError as e:
            return self.handleError("RecursiveFor: "+VenC.core.Messages.notEnoughArgs,"~§RecursiveFor§§"+"§§".join(argv)+"§~",True)
        except KeyError as e:
            return self.handleError(VenC.core.Messages.recursiveForUnknownValue.format(e),"~§RecursiveFor§§"+"§§".join(argv)+"§~",True)

    def preProcess(self, inputIndex, string):
        self.currentStrings[inputIndex] = str(string)
        self.preProcessedStrings[inputIndex] = list()
        self.patternsIndex[inputIndex] = list()
        closeSymbolPos	= list()
        openSymbolPos	= list()
        i		= int()
        
        while i < len(string):
            if i + len(self.openSymbol) <= len(string) and string[i:i+len(self.openSymbol)] == self.openSymbol:
                openSymbolPos.append(i)

            elif i + len(self.closeSymbol) <= len(string) and string[i:i+len(self.closeSymbol)] == self.closeSymbol:
                closeSymbolPos.append(i)

            if len(closeSymbolPos) == len(openSymbolPos) and len(closeSymbolPos) != 0 and len(openSymbolPos) != 0:
                self.preProcessedStrings[inputIndex].append(string[i-closeSymbolPos[-1]:openSymbolPos[0]])
                self.preProcessedStrings[inputIndex].append(string[openSymbolPos[0]:closeSymbolPos[-1]+len(self.closeSymbol)])
                self.patternsIndex[inputIndex].append(len(self.preProcessedStrings[inputIndex])-1)
                
                string = string[closeSymbolPos[-1]+len(self.closeSymbol):]
                openSymbolPos = list()
                closeSymbolPos = list()
                i=0
            else:
                i+=1

        self.preProcessedStrings[inputIndex].append(string)

    def parse(self, inputIndex, escape=False):
        if len(self.patternsIndex) == 0:
            return self.currentStrings[inputIndex]
        
        output = str()

        self.currentString = self.currentStrings[inputIndex]
        for index in range(0,len(self.preProcessedStrings[inputIndex])):
            if index in self.patternsIndex[inputIndex]:
                output += self._process(self.preProcessedStrings[inputIndex][index], escape)
            else:
                output += self.preProcessedStrings[inputIndex][index]

        return output

    def _process(self, string, escape):
        closeSymbolPos	= list()
        openSymbolPos	= list()
        output		= str()
        fields		= list()
        i		= int()


        while i < len(string):
            if i + len(self.openSymbol) <= len(string) and string[i:i+len(self.openSymbol)] == self.openSymbol:
                openSymbolPos.append(i)

            elif i + len(self.closeSymbol) <= len(string) and string[i:i+len(self.closeSymbol)] == self.closeSymbol:
                closeSymbolPos.append(i)

            if len(closeSymbolPos) <= len(openSymbolPos) and len(closeSymbolPos) != 0 and len(openSymbolPos) != 0:
                if openSymbolPos[-1] < closeSymbolPos[0]:
                    fields = [field for field in string[openSymbolPos[-1]+len(self.openSymbol):closeSymbolPos[0]].split(self.separator) if field != '']
                    if fields[0] in self.functions.keys():
                        output = self.functions[fields[0]](fields[1:])
                    else:
                        output =  self.handleError(VenC.core.Messages.unknownPattern.format(fields[0]),"~§"+"§§".join(fields)+"§~")
                    
                    if escape:
                        return self._process(string[:openSymbolPos[-1]]+cgi.escape(output).encode('ascii', 'xmlcharrefreplace').decode(encoding='ascii')+string[closeSymbolPos[0]+len(self.closeSymbol):],escape=True)
                    else:
                        return self._process(string[:openSymbolPos[-1]]+str(output)+string[closeSymbolPos[0]+2:],escape=False)

            i+=1
    
        return string



#!/usr/bin/python3

from setuptools import setup

setup(name='VenC',
    version='1.2.0-2',
    description='A static blog generator.',
    author='Denis Salem',
    author_email='denissalem@tuxfamily.org',
    url='https://github.com/DenisSalem/VenC',
    packages=['VenC', 'VenC.languages'],
    license="GNU/GPLv3",
    platforms="Linux",
    long_description="A static blog generator, light and easy to use.",
    classifiers=[
        "Environment :: Console",
        "Development Status :: 5 - Production/Stable"
    ],
    install_requires=[
          'markdown',
          'pyyaml',
          'Pygments'
    ],
    scripts=['venc','venc-unit-test']
)



#! /usr/bin/python3

import os
import time
import yaml
import shutil
import ftplib
import codecs
import getpass
import markdown
import subprocess
import VenC.core
import VenC.pattern

def ftpExportRecursively(origin, ftp):
        folder = os.listdir(origin)
        for item in folder:
            if os.path.isdir(origin+"/"+item):
                try:
                    ftp.mkd(item)
                    ftp.cwd(ftp.pwd()+"/"+item)
                    ftpExportRecursively(origin+"/"+item, ftp)
                    ftp.cwd(ftp.pwd()[:-len("/"+item)])
                except:
                    try:
                        ftp.cwd(ftp.pwd()+"/"+item)
                        ftpExportRecursively(origin+"/"+item, ftp)
                        ftp.cwd(ftp.pwd()[:-len("/"+item)])
                    except:
                        raise

            else:
                ftp.storbinary("STOR "+ftp.pwd()+"/"+item, open(origin+"/"+item, 'rb'))

def ftpCleanDestination(ftp):
    listing = list()
    listing = ftp.nlst()
    for item in listing:
        if item not in ['.','..']:
            try:
                ftp.delete(item)
            except Exception:
                try:
                    ftp.rmd(item)
                except:
                    ftp.cwd(ftp.pwd()+"/"+item)
                    ftpCleanDestination(ftp)
                    ftp.cwd(ftp.pwd()[:-len("/"+item)])

def remoteCopy(argv):
    if VenC.core.blogConfiguration == None:
        print("VenC: "+VenC.core.Messages.noBlogConfiguration)
        return

    try:
        ftp = ftplib.FTP(VenC.core.blogConfiguration["ftp_host"])
    except Exception as e:
        print("VenC:", e)
        return

    username = input("VenC: "+VenC.core.Messages.username)
    userPasswd = getpass.getpass(prompt="VenC: "+VenC.core.Messages.userPasswd)
    try:
        ftp.login(user=username,passwd=userPasswd)
        ftp.cwd(VenC.core.blogConfiguration["path"]["ftp"])
        print("VenC:", VenC.core.Messages.cleanFtpDirectory)
        ftpCleanDestination(ftp)
        print("VenC:", VenC.core.Messages.copyToFtpDirectory)
        ftpExportRecursively(os.getcwd()+"/blog", ftp)
    
    except ftplib.error_perm as e:
        print(e)
        return

def ftp(argv):
    print("VenC:", VenC.core.Messages.blogRecompilation)
    blog(argv)
    remoteCopy(argv)

def rmTreeErrorHandler(function, path, excinfo):
    if path == "blog" and excinfo[0] == FileNotFoundError:
      print("VenC: "+VenC.core.Messages.blogFolderDoesntExists)
      return

    print("VenC:",function)
    print("VenC:",path)
    print("VenC:",excinfo[0])
    exit()

def blog(argv):
    if VenC.core.blogConfiguration == None:
        print("VenC: "+VenC.core.Messages.noBlogConfiguration)
        return

    # cleaning direcoty
    shutil.rmtree("blog", ignore_errors=False, onerror=rmTreeErrorHandler)
    os.makedirs("blog")
    currentBlog = Blog()
    currentBlog.export()

def edit(argv):
    try:
        proc = subprocess.Popen([VenC.core.blogConfiguration["textEditor"], argv[0]])
        while proc.poll() == None:
            pass
    except:
        raise
    blog(list())

class Blog:
    def __init__(self):
        self.theme = VenC.core.Theme()
        self.entriesList = VenC.core.GetEntriesList()
        self.entriesPerDates = VenC.core.GetEntriesPerDates(self.entriesList)
        self.entriesPerCategories = VenC.core.GetEntriesPerCategories(self.entriesList)
        self.publicDataFromBlogConf = VenC.core.GetPublicDataFromBlogConf()
        self.entryCounter = 0
        self.pageCounter = 0
        self.outputPage = str()
        self.columns = list()
        self.inThread = True
        self.entry = dict()
        self.destinationPath = str()
        self.relativeOrigin = str()
        self.ressource = str()
        self.initPatternProcessor()

    def GetEntry(self, entryFilename, relativeOrigin=""):
        dump = yaml.load(self.entriesList[entryFilename].split("---\n")[0])
        if dump == None:
            return None

        output = dict()
        for key in dump.keys():
            if not key in ["authors","tags","categories","entry_name","doNotUseMarkdown"]:
                output["Entry"+key] = dump[key]
    

        # Optional since 1.2.0
        if "doNotUseMarkdown" in dump.keys():
            output["doNotUseMarkdown"] = True
        else:
            output["doNotUseMarkdown"] = False
   
        if "CSS" not in dump.keys():
            output["EntryCSS"] = ""

        output["EntryID"] = entryFilename.split('__')[0]
        output["EntryDate"] = VenC.core.GetFormattedDate(entryFilename.split('__')[1])

        try:
            output["EntryName"] = dump["entry_name"]
        except KeyError:
            print("VenC:",VenC.core.Messages.missingMandatoryFieldInEntry.format("entry_name", output["EntryID"]))
            exit()

        try:
            output["EntryAuthors"] = [ {"author":e} for e in list(dump["authors"].split(",") if dump["authors"] != str() else list()) ]
        except KeyError:
            print("VenC:",VenC.core.Messages.missingMandatoryFieldInEntry.format("authors", output["EntryID"]))
            exit()

        try:
            toBase64 = VenC.pattern.processor(".:",":.","::")
            toBase64.ressource = entryFilename
            toBase64.strict = False
            toBase64.SetFunction("CodeHighlight", VenC.core.ToBase64_)
            toBase64.preProcess(entryFilename,self.entriesList[entryFilename].split("---\n")[1])
            if output["doNotUseMarkdown"]:
                output["EntryContent"] = toBase64.parse(entryFilename)
            else:
                output["EntryContent"] = markdown.markdown( toBase64.parse(entryFilename) )
        except Exception as e:
            raise
            print("VenC:",VenC.core.Messages.possibleMalformedEntry.format(output["EntryID"]))
            exit()
        
        try:
            output["EntryTags"] = [ {"tag":e} for e in list(dump["tags"].split(",") if dump["tags"] != str() else list())]
        except KeyError:
            print("VenC:",VenC.core.Messages.missingMandatoryFieldInEntry.format("tags", output["EntryID"]))
            exit()
        except:
            output["EntryTags"] = list()
        try:
            entryPerCategories = GetEntriesPerCategories([entryFilename])
            output["EntryCategories"] = GetCategoriesTree(entryPerCategories, relativeOrigin, dict())
            output["EntryCategoriesLeaves"] = list()
            for category in dump["categories"].split(','):
                categoryLeaf= category.split(' > ')[-1].strip()
                if len(categoryLeaf) != 0:
                    categoryLeafUrl=str()
                    for subCategory in category.split(' > '):
                        categoryLeafUrl +=subCategory.strip()+'/'
                    output["EntryCategoriesLeaves"].append({"relativeOrigin":relativeOrigin, "categoryLeaf": categoryLeaf, "categoryLeafPath":categoryLeafUrl})
        except:
            output["EntryCategories"] = dict()
            output["EntryCategoriesLeaves"] = list()

        return output

    def IfInThread(self, argv):
        if self.inThread:
            return argv[0]
        else:
            return argv[1]

    def initPatternProcessor(self): 
        self.patternProcessor = VenC.pattern.processor(".:",":.","::")
        
        self.patternProcessor.preProcess("header", self.theme.header)
        self.patternProcessor.preProcess("entry", self.theme.entry)
        self.patternProcessor.preProcess("footer", self.theme.footer)
        self.patternProcessor.preProcess("rssHeader", self.theme.rssHeader)
        self.patternProcessor.preProcess("rssEntry", self.theme.rssEntry)
        self.patternProcessor.preProcess("rssFooter",self.theme.rssFooter)
        
        self.patternProcessor.SetFunction("IfInThread", self.IfInThread)
        self.patternProcessor.SetFunction("PagesList", self.GetPagesList)
        self.patternProcessor.SetFunction("CodeHighlight", VenC.core.CodeHighlight)

    def initStates(self, inputEntries, inThread):
        self.entryCounter = 0
        self.pageCounter = 0
        self.outputPage = str()
        self.inThread = inThread
        self.entriesNumber = len(inputEntries)

        categoriesTree = VenC.core.GetCategoriesTree(self.entriesPerCategories, self.relativeOrigin, dict(), maxWeight=VenC.core.GetCategoriesTreeMaxWeight(self.entriesPerCategories))
        
        self.patternProcessor.Set("PagesList", VenC.core.GetListOfPages(int(VenC.core.blogConfiguration["entries_per_pages"]),len(inputEntries)))
        self.patternProcessor.Set("BlogCategories", categoriesTree)
        self.patternProcessor.Set("BlogDates", VenC.core.GetDatesList(self.entriesPerDates, self.relativeOrigin))
        self.patternProcessor.Set("RelativeOrigin", self.relativeOrigin)
        self.patternProcessor.Set("RelativeLocation", self.destinationPath)
        self.patternProcessor.Set("SingleEntry", not inThread)
            
        if inThread:
            self.patternProcessor.SetFunction("GetPreviousPage", self.GetPreviousPageInThread)
            self.patternProcessor.SetFunction("GetNextPage", self.GetNextPageInThread)
        else:
            self.patternProcessor.SetFunction("GetPreviousPage", self.GetPreviousEntry)
            self.patternProcessor.SetFunction("GetNextPage", self.GetNextEntry)

        for key in self.publicDataFromBlogConf:
           self.patternProcessor.Set(key, self.publicDataFromBlogConf[key])

    def WritePage(self, folderDestination, entry):
        try:
            os.chdir("blog/")
            os.makedirs(folderDestination)
            os.chdir("../")
        except:
            os.chdir("../")
        if entry == -1:
            stream = codecs.open("blog/"+folderDestination+self.GetFilename(self.pageCounter-1),'w',encoding="utf-8")
        else:
            stream = codecs.open("blog/"+folderDestination+VenC.core.blogConfiguration["path"]["entry_file_name"].format(entry_id=entry),'w',encoding="utf-8")
        stream.write(self.outputPage)
        stream.close()

    def GetFilename(self, pageCounter):
        return VenC.core.blogConfiguration["path"]["index_file_name"].format(page_number=(str(pageCounter) if pageCounter != 0 else str()))

    def export(self):
        # Main thread
        print("VenC:",VenC.core.Messages.exportMainThread)
        self.exportThread(self.entriesList, True)
        # Entries
        self.exportThread(self.entriesList, False)
        
        self.relativeOrigin += "../"
        
        # Dates
        for e in self.entriesPerDates:
            self.relativeLocation = e.value+'/'
            print("VenC:", VenC.core.Messages.exportArchives.format(e.value))
            self.exportThread(e.relatedTo, True, folderDestination=e.value+'/')
        self.relativeOrigin = str()
        self.relativeLocation = str()
        
        # Categories
        self.exportCategories(self.entriesPerCategories)
        self.relativeOrigin = str()
        self.relativeLocation = str()
        
        print("VenC:",VenC.core.Messages.exportMainThreadRss)
        self.exportRss(self.entriesList)

        # Extra data
        self.exportExtraData(os.getcwd()+"/theme/assets")
        self.exportExtraData(os.getcwd()+"/extra")
    
    def exportExtraData(self,origin, destination=""):
        try:
            folder = os.listdir(origin)
            for item in folder:
                if os.path.isdir(origin+"/"+item):
                    try:
                        os.mkdir(os.getcwd()+"/blog/"+destination+item)
                        self.exportExtraData(origin+'/'+item, item+'/')
                    except:
                        raise
                else:
                    shutil.copy(origin+"/"+item, os.getcwd()+"/blog/"+destination+item)
        except:
            raise
        
    def exportCategories(self, categories):
        for category in categories:
            print("VenC:", VenC.core.Messages.exportCategories.format(category.value))
            self.destinationPath+= VenC.core.blogConfiguration["path"]["category_directory_name"].format(category=category.value+'/')
            self.relativeOrigin += "../"
            self.exportThread(category.relatedTo, True, folderDestination=self.destinationPath)
            print("VenC:",VenC.core.Messages.exportCategoriesRss.format(category.value))
            self.exportRss(category.relatedTo, folderDestination=self.destinationPath)
            self.exportCategories(category.childs)
            self.relativeOrigin = self.relativeOrigin[:-3]
            self.destinationPath = self.destinationPath[:-len(category.value+"/")]

    def GetPagesList(self, argv):
        try:
            listLenght = int(argv[0])
            pattern = argv[1]
            separator = argv[2]
            currentPage = self.patternProcessor.Get(["PageNumber"])
            pagesList = self.patternProcessor.Get(["PagesList"])
            
            if len(pagesList) == 1 or self.patternProcessor.Get(["SingleEntry"]):
                return str()

            output = str()
            for e in pagesList:
                if (not int(e["pageNumber"]) < int(currentPage) - listLenght) and (not int(e["pageNumber"]) > int(currentPage) + listLenght):
                    output += pattern.format(e) + separator

            return output[:-len(separator)]

        except Exception as e:
            return str(e)

    def GetPreviousEntry(self, argv):

        trigger = False
        output = dict()
        try:
            pattern = argv[0]
        except IndexError:
            return self.handleError("GetPreviousEntry: "+VenC.core.Messages.notEnoughArgs,"~§GetPreviousEntry§§"+"§§".join(argv)+"§~",True)

        sortedEntries = VenC.core.GetSortedEntriesList(self.entriesList.keys())
        for i in range(0, len(sortedEntries)):
            if trigger == True:
                output["destinationPageUrl"] = VenC.core.blogConfiguration["path"]["entry_file_name"].format(entry_id=sortedEntries[i].split("__")[0])
                output["destinationPage"] = sortedEntries[i].split("__")[0]
                output["entryName"] = self.GetEntry(sortedEntries[i])["EntryName"]
                return pattern.format(output)
            
            if sortedEntries[i].split("__")[0] == self.entry["EntryID"]:
                trigger = True;

        return str()

    def GetNextEntry(self, argv):

        trigger = False
        output = dict()
        try:
            pattern = argv[0]
        except IndexError:
            return self.handleError("GetPreviousEntry: "+VenC.core.Messages.notEnoughArgs,"~§GetNextEntry§§"+"§§".join(argv)+"§~",True)

        sortedEntries = list(reversed(VenC.core.GetSortedEntriesList(self.entriesList)))
        for i in range(0, len(sortedEntries)):
            if trigger == True:
                output["destinationPageUrl"] = VenC.core.blogConfiguration["path"]["entry_file_name"].format(entry_id=sortedEntries[i].split("__")[0])
                output["destinationPage"] = sortedEntries[i].split("__")[0]
                output["entryName"] = self.GetEntry(sortedEntries[i])["EntryName"]
                return pattern.format(output)
            
            if sortedEntries[i].split("__")[0] == self.entry["EntryID"]:
                trigger = True;

        return str()

    def GetNextPageInThread(self, argv):

        try:
            pattern = argv[0]
        except IndexError:
            return self.handleError("GetPreviousEntry: "+VenC.core.Messages.notEnoughArgs,"~§GetPreviousEntry§§"+"§§".join(argv)+"§~",True)
        currentPage = self.patternProcessor.Get(["PageNumber"])
        pagesCount = len(self.patternProcessor.Get(["PagesList"]))
        destinationPage = currentPage + 1
        destinationPageUrl = VenC.core.blogConfiguration["path"]["index_file_name"].format(page_number=str(destinationPage))
        if destinationPage > pagesCount - 1 or self.patternProcessor.Get(["SingleEntry"]):
            return str()
        else:
            try:
                return pattern.format({"destinationPage":destinationPage,"destinationPageUrl":destinationPageUrl,"entryName":""})
            except KeyError as e:
                err = VenC.core.OutputColors.FAIL+"VenC: GetNextPage: "+VenC.core.Messages.unknownContextual.format(e)+"\n"
                if self.ressource != str():
            	    err +="VenC: "+VenC.core.Messages.inRessource.format(self.ressource)+"\n"
                
                if not err in VenC.core.errors:
                    VenC.core.errors.append(err)
                    err += ".:GetNextPage::"+"::".join(argv)+":."+VenC.core.OutputColors.END+"\n\n"
                    print(err)
                return "<!-- ~§GetNextPage§§"+"§§".join(argv)+"§~ -->"
	        
    def GetPreviousPageInThread(self, argv):
        pattern = argv[0]
        currentPage = self.patternProcessor.Get(["PageNumber"])
        destinationPage = currentPage - 1
        destinationPageUrl = VenC.core.blogConfiguration["path"]["index_file_name"].format(page_number= ("" if currentPage - 1 == 0 else str(currentPage - 1)))
        if destinationPage < 0 or self.patternProcessor.Get(["SingleEntry"]):
            return str()
        else:
            try:
                return pattern.format({"destinationPage":destinationPage,"destinationPageUrl":destinationPageUrl,"entryName":""})
            except KeyError as e:
                err = VenC.core.OutputColors.FAIL+"VenC: GetPreviousPage: "+VenC.core.Messages.unknownContextual.format(e)+"\n"
                if self.ressource != str():
            	    err +="VenC: "+VenC.core.Messages.inRessource.format(self.ressource)+"\n"
                
                if not err in VenC.core.errors:
                    VenC.core.errors.append(err)
                    err += ".:GetPreviousPage::"+"::".join(argv)+":."+VenC.core.OutputColors.END+"\n\n"
                    print(err)
                return "<!-- ~§GetPreviousPage§§"+"§§".join(argv)+"§~ -->"

    def initEntryStates(self, entry):
        self.entry = VenC.core.MergeDictionnary(self.GetEntry(entry, self.relativeOrigin), self.publicDataFromBlogConf)
        
        self.entry["EntryContent"] = self.entry["EntryContent"].replace("~§",".:").replace("§§","::").replace("§~",":.")
        if self.entry == None:
            print("VenC:", VenC.core.Messages.possibleMalformedEntry.format(entry))
            exit()

        self.patternProcessor.Set("PageNumber", self.pageCounter)
        self.patternProcessor.Set("EntryDateUrl", self.relativeOrigin+time.strftime(VenC.core.blogConfiguration["path"]["dates_directory_name"], time.strptime(entry.split("__")[1],"%m-%d-%Y-%M-%S")))
        self.patternProcessor.Set("EntryUrl", self.relativeOrigin+"entry"+self.entry["EntryID"]+".html")
        self.patternProcessor.SetWholeDictionnary(self.entry)
        self.patternProcessor.ressource = str()
        self.ressource = str()

    def exportRss(self, inputEntries, folderDestination=""):
        self.initStates(inputEntries, True)
        self.outputPage = str()
        self.patternProcessor.ressource = "theme/chunks/rssHeader.html"
        self.ressource = "theme/chunks/rssHeader.html"
        self.outputPage += self.patternProcessor.parse("rssHeader")

        sortedEntries = VenC.core.GetSortedEntriesList(inputEntries)
        for entry in sortedEntries[:int(VenC.core.blogConfiguration["rss_thread_lenght"])]:
            self.initEntryStates(entry)
            self.patternProcessor.ressource = "theme/chunks/rssEntry.html"
            self.ressource = "theme/chunks/rssEntry.html"
            self.outputPage += self.patternProcessor.parse("rssEntry")
        self.patternProcessor.ressource = "theme/chunks/rssFooter.html"
        self.ressource = "theme/chunks/rssFooter.html"
        self.outputPage += self.patternProcessor.parse("rssFooter")
       	self.outputPage = self.outputPage.replace("<p><div","<div").replace("</div>\n</p>","</div>") # sanitize
       
        
        stream = codecs.open("blog/"+folderDestination+"/feed.xml",'w',encoding="utf-8")
        stream.write(self.outputPage)
        stream.close()

    def exportThread(self, inputEntries, inThread, folderDestination=""):
        self.initStates(inputEntries, inThread)

        # Configure patternProcessor instance with some fixed values and functions
        # Process actual entries
        if not inThread:
            columnsNumber = 1
            entries_per_pages = 1
        else:
            entries_per_pages  = VenC.core.blogConfiguration["entries_per_pages"]
            columnsNumber = 1 if VenC.core.blogConfiguration["columns"] < 1 else int(VenC.core.blogConfiguration["columns"])

        sortedEntries = VenC.core.GetSortedEntriesList(inputEntries)
        total = float()
        for entry in sortedEntries:
            self.initEntryStates(entry)
            if self.entryCounter == 0:
                self.columns = [ "<div id=\"__VENC_COLUMN_"+str(i)+"__\" class=\"__VENC_COLUMN__\">" for i in range(0,columnsNumber) ]
                self.outputPage = str()
                self.patternProcessor.ressource = "theme/chunks/header.html"
                self.ressource = "theme/chunks/header.html"
                self.outputPage += self.patternProcessor.parse("header")
           
            self.patternProcessor.ressource = entry
            self.ressource = entry
            self.columns[ self.entryCounter % columnsNumber ] += self.patternProcessor.parse("entry")+"\n"
            self.entryCounter += 1
            if self.entryCounter >= int(entries_per_pages) or entry == sortedEntries[-1]:
                self.columns = [ column+"</div>" for column in self.columns ]
                for column in self.columns:
                    self.outputPage += column
                self.patternProcessor.ressource = "theme/chunks/footer.html"
                self.ressource = "theme/chunks/footer.html"
                self.outputPage += self.patternProcessor.parse("footer")
       	        self.outputPage = self.outputPage.replace("<p><div","<div").replace("</div>\n</p>","</div>") #sanitize
                self.pageCounter += 1
                self.entryCounter = 0
                self.WritePage(folderDestination, ( int(entry.split("__")[0]) if not inThread else -1))
            
                

